(* absyn/absyn.sml *)

functor AbsynFn(structure MakeString : MAKESTRING
		structure Source : SOURCE
		  ) : ABSYN =
  struct

    structure Source	= Source

    datatype ident	= IDENT of string * int * int
    type var		= ident
    type tyvar		= ident

    datatype longid	= LONGID of ident option * ident

    datatype lit	= CCONlit of char
			| ICONlit of int
			| RCONlit of real
			| SCONlit of string

    datatype ty		= VARty of tyvar
			| CONSty of ty list * longid
			| TUPLEty of ty list
			| RELty of ty list * ty list

    datatype pat	= WILDpat
			| LITpat of lit
			| CONpat of longid
			| STRUCTpat of longid option * pat list
			| BINDpat of var * pat
			| IDENTpat of ident * pat ref

    datatype exp	= LITexp of lit
			| CONexp of longid
			| VARexp of longid
			| STRUCTexp of longid option * exp list
			| IDENTexp of longid * exp ref

    datatype goal	= CALLgoal of longid * exp list * pat list
			| EQUALgoal of var * exp
			| LETgoal of pat * exp
			| NOTgoal of goal
			| ANDgoal of goal * goal

    datatype result	= RETURN of exp list
			| FAIL
    datatype clause	= CLAUSE1 of goal option * ident * pat list * result
			| CLAUSE2 of clause * clause

    datatype conbind	= CONcb of ident
			| CTORcb of ident * ty list

    datatype datbind	= DATBIND of tyvar list * ident * conbind list

    datatype typbind	= TYPBIND of tyvar list * ident * ty

    datatype relbind	= RELBIND of ident * ty option * clause

    datatype spec	= WITHspec of string * interface ref
			| ABSTYPEspec of bool * tyvar list * ident
			| TYPEspec of typbind list
			| DATAspec of datbind list * typbind list
			| VALspec of ident * ty
			| RELspec of ident * ty

    and interface	= INTERFACE of {modid: ident,
					specs: spec list,
					source: Source.source}

    datatype dec	= WITHdec of string * interface ref
			| TYPEdec of typbind list
			| DATAdec of datbind list * typbind list
			| VALdec of ident * exp
			| RELdec of relbind list

    datatype module	= MODULE of interface * dec list

    structure IdentDict =
      AADictFn(struct
		 type ord_key = ident
		 fun compare(IDENT(s1,_,_), IDENT(s2,_,_)) =
		   String.compare(s1, s2)
	       end)

    fun makeIdent(name,left,right) = IDENT(name,left,right)
    fun rmlIdent name = IDENT(name,0,0)
    fun identName(IDENT(name,_,_)) = name
    fun identEqual(IDENT(s1,_,_), IDENT(s2,_,_)) = s1=s2

    fun litEqual(CCONlit c1, CCONlit c2) = c1 = c2
      | litEqual(ICONlit i1, ICONlit i2) = i1 = i2
      | litEqual(RCONlit r1, RCONlit r2) = Real.==(r1, r2)
      | litEqual(SCONlit s1, SCONlit s2) = s1 = s2
      | litEqual(_, _) = false

    fun litString(CCONlit c) = MakeString.ccvt c
      | litString(ICONlit i) = MakeString.icvt i
      | litString(RCONlit r) = MakeString.rcvt r
      | litString(SCONlit s) = MakeString.scvt s

    val dummyInterface =
      INTERFACE{modid = rmlIdent "", specs = [], source = Source.dummy}

  end (* functor AbsynFn *)
