(* main/link.sml *)

structure StrDict = AADictFn(StrKey);
structure IntDict = AADictFn(IntKey);
structure CPS =
  CPSFn(structure Util = Util
	 structure ConRep = ConRep);
structure CPSPrint =
  CPSPrintFn(structure MakeString = MakeString
	     structure Util = Util
	     structure PP = PP
	     structure CPS = CPS);
structure CPSUseful = CPSUsefulFn(structure Util = Util structure CPS = CPS);
structure CPSUsages = CPSUsagesFn(structure Util = Util structure CPS = CPS structure CPSUseful = CPSUseful);
structure CPSFVars = CPSFVarsFn(structure MakeString = MakeString structure Util = Util structure CPS = CPS);
structure CPSOptim =
  CPSOptimFn(structure Util = Util
	     structure CPS = CPS
	     structure CPSPrint = CPSPrint
	     structure CPSUsages = CPSUsages);

structure Absyn =
  AbsynFn(structure MakeString = MakeString
	  structure Source = Source
	    );
structure AbsynPrint =
  AbsynPrintFn(structure MakeString = MakeString
	       structure Util = Util
	       structure Absyn = Absyn);

structure LexArg = LexArgFn(structure Source = Source);
structure RMLLrVals =
  RMLLrValsFn(structure Token = LrParser.Token
	      structure Absyn = Absyn
	      structure LexArg = LexArg);
structure RMLLex =
  RMLLexFn(structure Tokens = RMLLrVals.Tokens
	   structure LexArg = LexArg
	   structure MakeString = MakeString
	   structure LexUtil = LexUtil);
structure RMLParser =
  JoinWithArg(structure Lex = RMLLex
	      structure ParserData = RMLLrVals.ParserData
	      structure LrParser = LrParser);
structure Parse =
  ParseFn(structure Absyn = Absyn
	  structure RMLParser = RMLParser
	  structure Tokens = RMLLrVals.Tokens
	  structure LexArg = LexArg);

structure ReorderTy =
  ReorderTyFn(structure Util = Util
              structure Absyn = Absyn
              structure ReorderSCC = ReorderSCC
		);
structure ReorderVal =
  ReorderValFn(structure Util = Util
               structure Absyn = Absyn
               structure ReorderSCC = ReorderSCC
		 );
structure Reorder =
  ReorderFn(structure ReorderTy = ReorderTy
            structure ReorderVal = ReorderVal);
structure ReorderParse =
  ReorderParseFn(structure Parse = Parse
                 structure Reorder = Reorder
		 structure Control = Control
		   );

structure FOL = FOLFn(structure Util = Util);
structure FOLUnify = FOLUnifyFn(structure FOL = FOL);
structure FOLUsages =
  FOLUsagesFn(structure Util = Util
	      structure FOL = FOL);
structure FOLPrint =
  FOLPrintFn(structure MakeString = MakeString
	     structure Util = Util
	     structure PP = PP
	     structure FOLUsages = FOLUsages);
structure FOLOptim =
  FOLOptimFn(structure Util = Util
	     structure FOLPrint = FOLPrint
	     structure FOLUnify = FOLUnify);

structure TyComb =
  TyCombFn(structure Util = Util
           structure Ty = Ty);
structure TyFcn =
  TyFcnFn(structure Util = Util
          structure TyComb = TyComb);
structure TyScheme =
  TySchemeFn(structure Util = Util
             structure TyComb = TyComb);
structure StatObj =
  StatObjFn(structure Absyn = Absyn
            structure TyFcn = TyFcn
            structure TyScheme = TyScheme);
structure StatElab =
  StatElabFn(structure Util = Util
             structure Parse = ReorderParse
             structure StatObj = StatObj
	     structure Control = Control
	       );

structure TransEnv =
  TransEnvFn(structure Util = Util
	     structure StrDict = StrDict
	     structure CPS = CPS);
structure AbsynToFOL =
  AbsynToFOLFn(structure StrDict = StrDict
	       structure Util = Util
	       structure Absyn = Absyn
	       structure FOL = FOL);
structure PMC =
  PMCFn(structure Util = Util
	structure CPS = CPS);
structure FOLToCPS =
  FOLToCPSFn(structure Util = Util
	     structure FOLUsages = FOLUsages
	     structure PMC = PMC
	     structure TransEnv = TransEnv
	     structure Control = Control
	       );
structure Mangle = MangleFn(structure Util = Util);

structure Plain =
  PlainFn(structure MakeString = MakeString
	  structure ConRep = ConRep
	  structure Mangle = Mangle);
structure PlainFVars =
  CodeFVarsFn(structure Util = Util
	      structure Code = Plain);
structure PlainOptim =
  CodeOptimFn(structure Util = Util
	      structure Code = Plain
	      structure CodeFVars = PlainFVars);
structure PlainCodeToC =
  CodeToCFn(structure MakeString = MakeString
	    structure Util = Util
	    structure Code = Plain
	    structure Control = Control
	      );
structure CPSToPlain =
  CPSToCodeFn(structure MakeString = MakeString
	      structure Util = Util
	      structure CPS = CPS
	      structure CPSUsages = CPSUsages
	      structure CPSFVars = CPSFVars
	      structure Code = Plain);
structure PlainToC =
  PlainToCFn(structure Util = Util
	     structure Code = Plain
	     structure CodeToC = PlainCodeToC);

structure Switch =
  SwitchFn(structure MakeString = MakeString
	   structure ConRep = ConRep
	   structure Mangle = Mangle);
structure SwitchFVars =
  CodeFVarsFn(structure Util = Util
	      structure Code = Switch);
structure SwitchOptim =
  CodeOptimFn(structure Util = Util
	      structure Code = Switch
	      structure CodeFVars = SwitchFVars);
structure SwitchCodeToC =
  CodeToCFn(structure MakeString = MakeString
	    structure Util = Util
	    structure Code = Switch
	    structure Control = Control
	      );
structure CPSToSwitch =
  CPSToCodeFn(structure MakeString = MakeString
	      structure Util = Util
	      structure CPS = CPS
	      structure CPSUsages = CPSUsages
	      structure CPSFVars = CPSFVars
	      structure Code = Switch);
structure SwitchIntras =
  SwitchIntrasFn(structure Util = Util
		 structure Code = Switch);

structure MaskToC =
  MaskToCFn(structure Code = Switch
	    structure SwitchIntras = SwitchIntras
	    structure CodeToC = SwitchCodeToC);

structure DiffToC =
  DiffToCFn(structure Code = Switch
	    structure SwitchIntras = SwitchIntras
	    structure CodeToC = SwitchCodeToC);

structure CPSToSML = CPSToSMLFn(structure CPS = CPS);

structure InterpCore =
  InterpCoreFn(structure MakeString = MakeString
	       structure LexUtil = LexUtil
	       structure Util = Util
	       structure Absyn = Absyn
	       structure IntDict = IntDict);
structure Interp =
  InterpFn(structure Util = Util
	   structure InterpCore = InterpCore);

structure Version = VersionFn(structure SysDep = SysDep);
structure Main =
  MainFn(structure Version = Version
	 structure Util = Util
	 structure Parse = ReorderParse
	 structure StatElab = StatElab
	 structure AbsynPrint = AbsynPrint
	 structure AbsynToFOL = AbsynToFOL
	 structure FOLPrint = FOLPrint
	 structure FOLOptim = FOLOptim
	 structure FOLToCPS = FOLToCPS
	 structure CPSOptim = CPSOptim
	 structure CPSPrint = CPSPrint
	 structure CPSToSwitch = CPSToSwitch
	 structure SwitchOptim = SwitchOptim
	 structure CPSToPlain = CPSToPlain
	 structure PlainOptim = PlainOptim
	 structure PlainToC = PlainToC
	 structure MaskToC = MaskToC
	 structure DiffToC = DiffToC
	 structure CPSToSML = CPSToSML
	 structure Interp = Interp
	 structure Control = Control
	   );
